#ifndef MARS_MCalibrationHiLoPix
#define MARS_MCalibrationHiLoPix

#ifndef MARS_MCalibrationPix
#include "MCalibrationPix.h"
#endif

class MCalibrationHiLoPix : public MCalibrationPix
{
private:

public:

  MCalibrationHiLoPix(const char *name=NULL, const char *title=NULL);
  ~MCalibrationHiLoPix() {}
  
  Float_t GetHiLoChargeRatio()         const { return GetHiGainMean();     }
  Float_t GetHiLoChargeRatioErr()      const { return GetHiGainMeanErr();  }
  Float_t GetHiLoChargeRatioSigma()    const { return GetHiGainSigma();    }
  Float_t GetHiLoChargeRatioSigmaErr() const { return GetHiGainSigmaErr(); }
  Float_t GetHiLoChargeRatioProb()     const { return GetHiGainProb();     }
									   
  Float_t GetHiLoTimeDiff()            const { return GetLoGainMean();     }
  Float_t GetHiLoTimeDiffErr()         const { return GetLoGainMeanErr();  }
  Float_t GetHiLoTimeDiffSigma()       const { return GetLoGainSigma();    }
  Float_t GetHiLoTimeDiffSigmaErr()    const { return GetLoGainSigmaErr(); }
  Float_t GetHiLoTimeDiffProb()        const { return GetLoGainProb();     }

  ClassDef(MCalibrationHiLoPix, 1)	// Container HiLo conversion Calibration Results Pixel
};

#endif

